@@ -2,7 +2,7 @@
  *  Hamlib Kenwood backend - TS570 description
  *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: ts570.c,v 1.23 2005/04/03 20:14:26 fillods Exp $
+ *	$Id: ts570.c,v 1.31 2006/11/29 19:38:00 y32kn Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -26,9 +26,11 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <hamlib/rig.h>
 #include "kenwood.h"
+#include "ic10.h"
 
 
 #define TS570_ALL_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_SSB|RIG_MODE_FM|RIG_MODE_RTTY)
@@ -37,7 +39,7 @@
 
 #define TS570_FUNC_ALL (RIG_FUNC_FAGC|RIG_FUNC_TSQL|RIG_FUNC_TONE|RIG_FUNC_NB|RIG_FUNC_COMP|RIG_FUNC_VOX|RIG_FUNC_NR|RIG_FUNC_LOCK|RIG_FUNC_BC|RIG_FUNC_TUNER)
 
-#define TS570_LEVEL_ALL (RIG_LEVEL_ATT|RIG_LEVEL_AGC|RIG_LEVEL_SQL|RIG_LEVEL_STRENGTH|RIG_LEVEL_AF|RIG_LEVEL_RF|RIG_LEVEL_RFPOWER|RIG_LEVEL_MICGAIN)
+#define TS570_LEVEL_ALL (RIG_LEVEL_PREAMP|RIG_LEVEL_ATT|RIG_LEVEL_AGC|RIG_LEVEL_SQL|RIG_LEVEL_STRENGTH|RIG_LEVEL_AF|RIG_LEVEL_RF|RIG_LEVEL_RFPOWER|RIG_LEVEL_MICGAIN)
 
 #define TS570_VFO (RIG_VFO_A|RIG_VFO_B)
 #define TS570_VFO_OP (RIG_OP_UP|RIG_OP_DOWN)
@@ -62,10 +64,28 @@ static const struct kenwood_priv_caps  ts570_priv_caps  = {
 		.cmdtrm =  EOM_KEN,
 };
 
+
+static rmode_t char_to_mode(char c)
+{
+        switch (c) {
+            case MD_CW:     return(RIG_MODE_CW);
+            case MD_CWR:    return(RIG_MODE_CWR);
+            case MD_USB:    return(RIG_MODE_USB);
+            case MD_LSB:    return(RIG_MODE_LSB);
+            case MD_FM:     return(RIG_MODE_FM);
+            case MD_AM:     return(RIG_MODE_AM);
+            case MD_FSK:    return(RIG_MODE_RTTY);
+            case MD_FSKR:   return(RIG_MODE_RTTYR);
+            case MD_NONE:   return(RIG_MODE_NONE);
+	}
+        return(RIG_MODE_NONE);
+}
+
 static int ts570_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
-  unsigned char buf[50];
-  int buf_len, retval;
+  char buf[50];
+  size_t buf_len;
+  int retval;
 
 
   buf_len = 50;
@@ -142,33 +162,38 @@ static int ts570_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
   return RIG_OK;
 }
 
+static char mode_to_char(rmode_t mode)
+{
+      switch (mode) {
+         case RIG_MODE_CW:       return(MD_CW);
+         case RIG_MODE_CWR:      return(MD_CWR);
+         case RIG_MODE_USB:      return(MD_USB);
+         case RIG_MODE_LSB:      return(MD_LSB);
+         case RIG_MODE_FM:       return(MD_FM);
+         case RIG_MODE_AM:       return(MD_AM);
+         case RIG_MODE_RTTY:     return(MD_FSK);
+         case RIG_MODE_RTTYR:    return(MD_FSKR);
+	 default:
+		rig_debug(RIG_DEBUG_WARN,"ts570_mode_to_char %s: unsupported mode %d\n", __FUNCTION__,mode);
+     }
+ return(RIG_MODE_NONE);
+}
 
 static int ts570_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
-  unsigned char buf[16],ackbuf[16];
-  int buf_len, ack_len, kmode, retval;
+  char buf[16],ackbuf[16];
+  int buf_len, kmode, retval;
+  size_t ack_len;
 
-  switch (mode) 
-  {
-    case RIG_MODE_CW:       kmode = MD_CW; break;
-    case RIG_MODE_CWR:      kmode = MD_CWR; break;
-    case RIG_MODE_USB:      kmode = MD_USB; break;
-    case RIG_MODE_LSB:      kmode = MD_LSB; break;
-    case RIG_MODE_FM:       kmode = MD_FM; break;
-    case RIG_MODE_AM:       kmode = MD_AM; break;
-    case RIG_MODE_RTTY:     kmode = MD_FSK; break;
-    case RIG_MODE_RTTYR:    kmode = MD_FSKR; break;
-    default:
-      rig_debug(RIG_DEBUG_ERR,"ts570_set_mode: "
-	"unsupported mode %d\n", mode);
-      return -RIG_EINVAL;
-  }
+  if ((kmode = mode_to_char(mode)) == RIG_MODE_NONE)
+     return -RIG_EINVAL;
 
   buf_len = sprintf(buf, "MD%c;", kmode);
   ack_len = 0;
   retval = kenwood_transaction (rig, buf, buf_len, ackbuf, &ack_len);
   if (retval != RIG_OK) return retval;
 
+  ack_len = 0;
   switch (mode) 
   {
     case RIG_MODE_CW:
@@ -176,7 +201,6 @@ static int ts570_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     case RIG_MODE_RTTY:
     case RIG_MODE_RTTYR:
       buf_len = sprintf(buf, "FW%04d;", (int)width);
-      ack_len = 0;
       retval = kenwood_transaction (rig, buf, buf_len, ackbuf, &ack_len);
       if (retval != RIG_OK) return retval;
       break;
@@ -185,7 +209,6 @@ static int ts570_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
     case RIG_MODE_FM:
     case RIG_MODE_AM:
       buf_len = sprintf(buf, "SL%02d;", (int)width/50);
-      ack_len = 0;
       retval = kenwood_transaction (rig, buf, buf_len, ackbuf, &ack_len);
       if (retval != RIG_OK) return retval;
       break;
@@ -202,11 +225,12 @@ static int ts570_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
  */
 static int ts570_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
 {
-	unsigned char buf[6], ackbuf[16];
-	int len, ack_len, retval;
+	char buf[6], ackbuf[16];
+	int len, retval;
+	size_t ack_len;
 
 	len = sprintf(buf,"AN%c;", ant==RIG_ANT_1?'1':'2');
-
+	ack_len = 0;
 	retval = kenwood_transaction(rig, buf, len, ackbuf, &ack_len);
 
 	return retval;
@@ -219,10 +243,11 @@ static int ts570_set_ant(RIG *rig, vfo_t vfo, ant_t ant)
  */
 static int ts570_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 {
-	unsigned char infobuf[50];
-	int info_len, retval;
+	char infobuf[50];
+	int retval;
+	size_t info_len;
 
-	info_len = 5;
+	info_len = 50;
 	retval = kenwood_transaction (rig, "AN;", 3, infobuf, &info_len);
 	if (retval != RIG_OK)
 		return retval;
@@ -238,7 +263,360 @@ static int ts570_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
 	return RIG_OK;
 }
 
+/*
+ * extends kenwood_set_func
+ * Assumes rig!=NULL, val!=NULL
+ */
+int ts570_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
+{
+		char fctbuf[16], ackbuf[16];
+		int fct_len;
+		size_t ack_len;
 
+		/* Filter unimplemented RIG_FUNC_TUNER and allow settings 0..2 for 
+		 * RIG_FUNC_NR.
+		 * Send all other requests to kenwood_set_func()
+		 */
+		ack_len = 0;
+		switch (func) {
+
+		case RIG_FUNC_NR:
+			if ((status < 0) || (status >2))
+				return -RIG_EINVAL;
+			fct_len = sprintf(fctbuf,"NR%01d;", status);
+			return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+		case RIG_FUNC_TUNER:
+			fct_len = sprintf(fctbuf,"AC %c0;", (0==status)?'0':'1');
+ 			return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+		
+		default:
+			return kenwood_set_func(rig, vfo, func, status);
+		}
+
+		return RIG_OK;
+}
+
+
+/*
+ * extends kenwood_get_func
+ * Assumes rig!=NULL, val!=NULL
+ */
+int ts570_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
+{
+		char fctbuf[50];
+		size_t fct_len;
+		int retval;
+
+		fct_len = 50;
+		/* filter unimplemented RIG_FUNC_TUNER  
+		 * and send all other requests to kenwood_get_func()
+		 */
+		switch (func) {
+		case RIG_FUNC_NR:
+			retval = kenwood_transaction (rig, "NR;", 3, fctbuf, &fct_len);
+			if (retval != RIG_OK)
+				return retval;
+
+			if (fct_len != 4) {
+				rig_debug(RIG_DEBUG_ERR,"kenwood_get_func: " 
+					"wrong answer len=%d\n", fct_len);
+				return -RIG_ERJCTED;
+			}
+			
+			*status = atoi(&fctbuf[2]);
+			break;
+
+		case RIG_FUNC_TUNER:
+			retval = kenwood_transaction (rig, "AC;", 3, fctbuf, &fct_len);
+			if (retval != RIG_OK)
+				return retval;
+
+			if (fct_len != 6) {
+				rig_debug(RIG_DEBUG_ERR,"kenwood_get_func: " 
+					"wrong answer len=%d\n", fct_len);
+				return -RIG_ERJCTED;
+			}
+
+			*status = fctbuf[3] == '0' ? 0 : 1;
+			break;
+
+		default:
+			return kenwood_get_func(rig, vfo, func, status);
+		}
+
+		return RIG_OK;
+}
+
+/*
+ * ts570_set_level
+ * Assumes rig!=NULL
+ *
+ * set levels of most functions
+ */
+int
+ts570_set_level (RIG * rig, vfo_t vfo, setting_t level, value_t val)
+{
+  char levelbuf[16], ackbuf[16];
+  int level_len;
+  size_t ack_len;
+  int kenwood_val;
+  int i;
+
+  ack_len = 0;			/* no answer for set functions */
+
+  switch (level)
+    {
+    case RIG_LEVEL_PREAMP:
+        kenwood_val = val.i;
+	/* set the preamplifier if a correct value is entered */
+	if (kenwood_val == 0)
+		level_len = sprintf(levelbuf, "PA0;");
+	else
+	for (i=0; i<MAXDBLSTSIZ; i++)
+	    if (kenwood_val == rig->state.preamp[i])
+	    {
+		level_len = sprintf(levelbuf, "PA%01d;", i+1);
+		break;  /* found - stop searching */
+	    }
+	    else
+	    	return -RIG_EINVAL;
+        return kenwood_transaction (rig, levelbuf, level_len, ackbuf, &ack_len);
+
+    case RIG_LEVEL_RFPOWER:
+      /* level for TS570D is from 0.. 100W in SSB and CW */
+      kenwood_val = val.f * 100;	
+      level_len = sprintf (levelbuf, "PC%03d;", kenwood_val);
+      return kenwood_transaction (rig, levelbuf, level_len, ackbuf, &ack_len);
+      
+    case RIG_LEVEL_MICGAIN:
+      /* level is from 0..100 */
+      kenwood_val = val.f * 100;
+      level_len = sprintf (levelbuf, "MG%03d;", kenwood_val);
+      return kenwood_transaction (rig, levelbuf, level_len, ackbuf, &ack_len);
+
+    default:
+      return kenwood_set_level (rig, vfo, level, val);
+    }
+  
+  return RIG_OK;		/* never reached */
+}
+
+
+/*
+ * ts570_get_level
+ * Assumes rig!=NULL, val!=NULL
+ */
+int
+ts570_get_level (RIG * rig, vfo_t vfo, setting_t level, value_t * val)
+{
+  char ackbuf[50];
+  size_t ack_len = 50;
+  int levelint;
+  int retval;
+  int i;
+
+  switch (level)
+    {
+    case RIG_LEVEL_RFPOWER:
+      /* ts570d returns 5..100 measured in watt */
+      retval = kenwood_transaction (rig, "PC;", 3, ackbuf, &ack_len);
+      if (RIG_OK != retval)
+	return retval;
+      if (6 != ack_len)
+	return -RIG_EPROTO;
+      if (1 != sscanf (&ackbuf[2], "%d", &levelint))
+	return -RIG_EPROTO;
+      val->f = (float) levelint / 100.;
+      return RIG_OK;
+
+    case RIG_LEVEL_MICGAIN:
+      /* reads from 0..100 */
+      retval = kenwood_transaction (rig, "MG;", 3, ackbuf, &ack_len);
+      if (RIG_OK != retval)
+        return retval;
+      if (6 != ack_len)
+        return -RIG_EPROTO;
+      if (1 != sscanf (&ackbuf[2], "%d", &levelint))
+	return -RIG_EPROTO;
+      val->f = (float) levelint / 100.;
+      return RIG_OK;
+
+    case RIG_LEVEL_PREAMP:
+	retval = kenwood_transaction (rig, "PA;", 3, ackbuf, &ack_len);
+	if (retval != RIG_OK)
+	    return retval;
+
+	if (4  != ack_len) {
+	    rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer len=%d\n",
+			__FUNCTION__, ack_len);
+	    return -RIG_ERJCTED;
+	}
+
+	sscanf(ackbuf+2, "%d", &levelint);
+	if (levelint == 0) {
+	    val->i = 0;
+	} else {
+	    for (i=0; i<levelint && i<MAXDBLSTSIZ; i++)
+		if (rig->state.preamp[i] == 0) {
+		    rig_debug(RIG_DEBUG_ERR,"%s: "
+			    "unexpected att level %d\n",
+			    __FUNCTION__, levelint);
+		    return -RIG_EPROTO;
+		}
+		if (i != levelint)
+		    return -RIG_EINTERNAL;
+		val->i = rig->state.preamp[i-1];
+	}
+	break;
+    default:
+ 	return kenwood_get_level (rig, vfo, level, val);
+    }
+  
+  return RIG_OK;		/* never reached */
+}
+
+
+/* memory capabilities */
+#define TS570_MEM_CAP {		\
+	.freq = 1,		\
+	.mode = 1,		\
+	.tx_freq=1,		\
+	.tx_mode=1,		\
+	.split=1,		\
+	.ctcss_tone=1		\
+}
+
+/* 
+ * read memory channel
+ * Assumes: rig != NULL, chan != NULL
+ * 
+ * !!! memory channel remembers additional settings for antenna, attenuator and
+ * !!! preamplifier and ctcss squelch. NOT SUPPORTED YET!
+ */
+int ts570_get_channel (RIG * rig, channel_t * chan)
+{
+                char cmdbuf[16], membuf[50];
+                int retval, cmd_len, num;
+		size_t mem_len;
+
+		/* 
+		 * first check if we shall get a memory or vfo channel
+		 */
+		if (chan->vfo != RIG_VFO_MEM) {
+			/*
+			 * At the time only memory channels are supported. 
+			 */
+			rig_debug(RIG_DEBUG_ERR,"kenwood_get_chan: " 
+					"ts570_get_channel not implemented"
+					" for VFO channels.\n");
+			return -RIG_ENIMPL;
+		}
+
+		num=chan->channel_num;
+
+                cmd_len = sprintf(cmdbuf, "MR0 %02d;", num);
+                mem_len = 50;
+                retval = kenwood_transaction (rig, cmdbuf, cmd_len, membuf, &mem_len);
+                if (retval != RIG_OK)
+                                return retval;
+
+		memset(chan,0,sizeof(channel_t));
+		chan->channel_num=num;
+		
+
+		if (membuf[19]=='0')
+			chan->ctcss_tone=0;
+		else {
+			membuf[22]='\0';
+			chan->ctcss_tone= rig->caps->ctcss_list[atoi(&membuf[20])-1];
+		}
+
+		chan->mode=char_to_mode(membuf[17]);
+
+		membuf[17]='\0';
+		chan->freq=atoi(&membuf[6]);
+
+		/* See if channel is in use */
+		if (chan->freq == RIG_FREQ_NONE)
+			return -RIG_ENAVAIL;
+
+		/* Manual says that we can read the scanning stop frequency
+		 * for channel 90..99 by the following command.
+		 * That seems not to be true. It returns the same information 
+		 * as MR0.
+		 * So we use it here only for getting the split info for
+		 * channels 00..89.
+		 */
+                cmd_len = sprintf(cmdbuf, "MR1 %02d;", num);
+                mem_len = 50;
+                retval = kenwood_transaction (rig, cmdbuf, cmd_len, membuf, &mem_len);
+                if (retval != RIG_OK)
+                                return retval;
+
+		chan->tx_mode=char_to_mode(membuf[17]);
+		membuf[17]='\0';
+		chan->tx_freq=atoi(&membuf[6]);
+
+		if(chan->freq==chan->tx_freq &&
+		   chan->mode==chan->tx_mode) {
+			chan->tx_freq=RIG_FREQ_NONE;
+			chan->tx_mode=RIG_MODE_NONE;
+			chan->split=RIG_SPLIT_OFF;
+		} else
+		        chan->split=RIG_SPLIT_ON;
+
+		return RIG_OK;
+}
+
+
+int ts570_set_channel (RIG * rig, const channel_t * chan)
+{
+                char cmdbuf[30], membuf[30];
+                int retval, cmd_len;
+		size_t mem_len;
+		int num,freq,tx_freq,tone;
+		char mode,tx_mode,split,tones;
+
+		num=chan->channel_num;
+		freq=(int)chan->freq;
+		mode=mode_to_char(chan->mode);
+		if(chan->split==RIG_SPLIT_ON) {
+			tx_freq=(int)chan->tx_freq;
+			tx_mode=mode_to_char(chan->tx_mode);
+			split='1';
+		} else {
+			tx_freq=0;
+			tx_mode='\0';
+			split='0';
+		}
+
+        	for (tone = 1; rig->caps->ctcss_list[tone-1] != 0 && tone<39; tone++) {
+                        if (rig->caps->ctcss_list[tone-1] == chan->ctcss_tone)
+                                        break;
+        	}
+		if(chan->ctcss_tone!=0) {
+			tones='1';
+		} else {
+			tones='0';
+			tone=0;
+		}
+
+                cmd_len = sprintf(cmdbuf, "MW0 %02d%011d%c0%c%02d ;",
+		 num,freq,mode,tones,tone);
+                mem_len = 0;
+                retval = kenwood_transaction (rig, cmdbuf, cmd_len, membuf, &mem_len);
+                if (retval != RIG_OK)
+                                return retval;
+
+                cmd_len = sprintf(cmdbuf, "MW1 %02d%011d%c0%c%02d ;",
+		 num,tx_freq,tx_mode,tones,tone);
+                mem_len = 0;
+                retval = kenwood_transaction (rig, cmdbuf, cmd_len, membuf, &mem_len);
+                if (retval != RIG_OK)
+                                return retval;
+
+		return RIG_OK;
+}
 /*
  * ts570 rig capabilities.
  * Notice that some rigs share the same functions.
@@ -265,8 +643,8 @@ const struct rig_caps ts570s_caps = {
 .serial_parity =  RIG_PARITY_NONE,
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
-.post_write_delay =  0,
-.timeout =  200,
+.post_write_delay =  50,
+.timeout = 1000,
 .retry =  3,
 
 .has_get_func =  TS570_FUNC_ALL,
@@ -279,7 +657,7 @@ const struct rig_caps ts570s_caps = {
 .parm_gran =  {},
 .ctcss_list =  kenwood38_ctcss_list,
 .dcs_list =  NULL,
-.preamp =   { RIG_DBLST_END, },	/* FIXME: preamp list */
+.preamp =   { 12, RIG_DBLST_END, },
 .attenuator =   { 18, RIG_DBLST_END, },
 .max_rit =  Hz(9990),
 .max_xit =  Hz(9990),
@@ -293,8 +671,8 @@ const struct rig_caps ts570s_caps = {
 
 
 .chan_list =  {
-			{  0, 89, RIG_MTYPE_MEM  },
-			{ 90, 99, RIG_MTYPE_EDGE },
+			{  0, 89, RIG_MTYPE_MEM,  TS570_MEM_CAP },
+			{ 90, 99, RIG_MTYPE_EDGE, TS570_MEM_CAP },
 		  	RIG_CHAN_END,
 		   },
 .rx_range_list1 =  { 
@@ -379,7 +757,7 @@ const struct rig_caps ts570s_caps = {
 .priv =  (void *)&ts570_priv_caps,
 
 .set_freq =  kenwood_set_freq,
-.get_freq =  kenwood_get_freq,
+.get_freq =  ic10_get_freq,
 .set_rit =  kenwood_set_rit,
 .get_rit =  kenwood_get_rit,
 .set_xit =  kenwood_set_xit,
@@ -393,21 +771,23 @@ const struct rig_caps ts570s_caps = {
 .get_ptt =  kenwood_get_ptt,
 .set_ptt =  kenwood_set_ptt,
 .get_dcd =  kenwood_get_dcd,
-.set_func =  kenwood_set_func,
-.get_func =  kenwood_get_func,
+.set_func =  ts570_set_func,
+.get_func =  ts570_get_func,
 .set_ant =  ts570_set_ant,
 .get_ant =  ts570_get_ant,
-.set_level =  kenwood_set_level,
-.get_level =  kenwood_get_level,
+.set_level =  ts570_set_level,
+.get_level =  ts570_get_level,
 .send_morse =  kenwood_send_morse,
 .vfo_op =  kenwood_vfo_op,
 .set_mem =  kenwood_set_mem,
 .get_mem =  kenwood_get_mem,
+.get_channel = ts570_get_channel,
+.set_channel = ts570_set_channel,
 .set_trn =  kenwood_set_trn,
-.scan =  kenwood_scan,
 .get_trn =  kenwood_get_trn,
 .set_powerstat =  kenwood_set_powerstat,
 .get_powerstat =  kenwood_get_powerstat,
+.scan =  kenwood_scan,
 .reset =  kenwood_reset,
 
 };
@@ -426,7 +806,7 @@ const struct rig_caps ts570d_caps = {
 .mfg_name =  "Kenwood",
 .version =  BACKEND_VER,
 .copyright =  "LGPL",
-.status =  RIG_STATUS_UNTESTED,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_RIG,
 .dcd_type =  RIG_DCD_RIG,
@@ -438,8 +818,8 @@ const struct rig_caps ts570d_caps = {
 .serial_parity =  RIG_PARITY_NONE,
 .serial_handshake =  RIG_HANDSHAKE_NONE,
 .write_delay =  0,
-.post_write_delay =  0,
-.timeout =  200,
+.post_write_delay =  50,
+.timeout =  1000,
 .retry =  3,
 
 .has_get_func =  TS570_FUNC_ALL,
@@ -452,7 +832,7 @@ const struct rig_caps ts570d_caps = {
 .parm_gran =  {},
 .ctcss_list =  kenwood38_ctcss_list,
 .dcs_list =  NULL,
-.preamp =   { RIG_DBLST_END, },	/* FIXME: preamp list */
+.preamp =   { 12, RIG_DBLST_END, },
 .attenuator =   { 18, RIG_DBLST_END, },
 .max_rit =  Hz(9990),
 .max_xit =  Hz(9990),
@@ -465,59 +845,59 @@ const struct rig_caps ts570d_caps = {
 .chan_desc_sz =  0,
 
 .chan_list =  {
-			{  0, 89, RIG_MTYPE_MEM  },
-			{ 90, 99, RIG_MTYPE_EDGE },
+			{  0, 89, RIG_MTYPE_MEM,  TS570_MEM_CAP  },
+			{ 90, 99, RIG_MTYPE_EDGE, TS570_MEM_CAP  },
 		  	RIG_CHAN_END,
 		   },
 .rx_range_list1 =  { 
-	{kHz(500),MHz(30),TS570_ALL_MODES,-1,-1,TS570_VFO},
+	{kHz(500),MHz(30),TS570_ALL_MODES,-1,-1,TS570_VFO,TS570_ANTS},
 	RIG_FRNG_END,
   }, /* rx range */
 .tx_range_list1 =  {
-    {kHz(1810),kHz(1850)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},	/* 100W class */
-    {kHz(1800),MHz(2)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO},		/* 25W class */
-    {kHz(3500),kHz(3800)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(3500),kHz(3800)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(7),kHz(7100),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(7),kHz(7100),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(10100),kHz(10150),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(10100),kHz(10150),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(14),kHz(14350),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(14),kHz(14350),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(18068),kHz(18168),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(18068),kHz(18168),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(21),kHz(21450),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(21),kHz(21450),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(24890),kHz(24990),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(24890),kHz(24990),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(28),kHz(29700),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(28),kHz(29700),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
+    {kHz(1810),kHz(1850)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},	/* 100W class */
+    {kHz(1800),MHz(2)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},		/* 25W class */
+    {kHz(3500),kHz(3800)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(3500),kHz(3800)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(7),kHz(7100),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(7),kHz(7100),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(10100),kHz(10150),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(10100),kHz(10150),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(14),kHz(14350),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(14),kHz(14350),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(18068),kHz(18168),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(18068),kHz(18168),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(21),kHz(21450),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(21),kHz(21450),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(24890),kHz(24990),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(24890),kHz(24990),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(28),kHz(29700),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(28),kHz(29700),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
 	RIG_FRNG_END,
   }, /* tx range */
 
 .rx_range_list2 =  {
-	{kHz(500),MHz(30),TS570_ALL_MODES,-1,-1,TS570_VFO},
+	{kHz(500),MHz(30),TS570_ALL_MODES,-1,-1,TS570_VFO,TS570_ANTS},
 	RIG_FRNG_END,
   }, /* rx range */
 .tx_range_list2 =  {
-    {kHz(1800),MHz(2)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},	/* 100W class */
-    {kHz(1800),MHz(2)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO},		/* 25W class */
-    {kHz(3500),MHz(4)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(3500),MHz(4)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(7),kHz(7300),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(7),kHz(7300),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(10100),kHz(10150),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(10100),kHz(10150),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(14),kHz(14350),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(14),kHz(14350),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(18068),kHz(18168),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(18068),kHz(18168),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(21),kHz(21450),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(21),kHz(21450),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {kHz(24890),kHz(24990),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {kHz(24890),kHz(24990),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
-    {MHz(28),kHz(29700),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO},
-    {MHz(28),kHz(29700),TS570_AM_TX_MODES,5000,25000,TS570_VFO},
+    {kHz(1800),MHz(2)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},	/* 100W class */
+    {kHz(1800),MHz(2)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},		/* 25W class */
+    {kHz(3500),MHz(4)-1,TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(3500),MHz(4)-1,TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(7),kHz(7300),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(7),kHz(7300),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(10100),kHz(10150),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(10100),kHz(10150),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(14),kHz(14350),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(14),kHz(14350),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(18068),kHz(18168),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(18068),kHz(18168),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(21),kHz(21450),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(21),kHz(21450),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {kHz(24890),kHz(24990),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {kHz(24890),kHz(24990),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
+    {MHz(28),kHz(29700),TS570_OTHER_TX_MODES,5000,100000,TS570_VFO,TS570_ANTS},
+    {MHz(28),kHz(29700),TS570_AM_TX_MODES,5000,25000,TS570_VFO,TS570_ANTS},
 	RIG_FRNG_END,
   }, /* tx range */
 .tuning_steps =  {
@@ -557,13 +937,13 @@ const struct rig_caps ts570d_caps = {
 .priv =  (void *)&ts570_priv_caps,
 
 .set_freq =  kenwood_set_freq,
-.get_freq =  kenwood_get_freq,
+.get_freq =  ic10_get_freq,
 .set_rit =  kenwood_set_rit,
 .get_rit =  kenwood_get_rit,
 .set_xit =  kenwood_set_xit,
 .get_xit =  kenwood_get_xit,
-.set_mode =  kenwood_set_mode,
-.get_mode =  kenwood_get_mode,
+.set_mode =  ts570_set_mode,
+.get_mode =  ts570_get_mode,
 .set_vfo =  kenwood_set_vfo,
 .get_vfo =  kenwood_get_vfo,
 .set_ctcss_tone =  kenwood_set_ctcss_tone,
@@ -571,14 +951,18 @@ const struct rig_caps ts570d_caps = {
 .get_ptt =  kenwood_get_ptt,
 .set_ptt =  kenwood_set_ptt,
 .get_dcd =  kenwood_get_dcd,
-.set_func =  kenwood_set_func,
-.get_func =  kenwood_get_func,
-.set_level =  kenwood_set_level,
-.get_level =  kenwood_get_level,
+.set_func =  ts570_set_func,
+.get_func =  ts570_get_func,
+.set_ant =  ts570_set_ant,
+.get_ant =  ts570_get_ant,
+.set_level =  ts570_set_level,
+.get_level =  ts570_get_level,
 .send_morse =  kenwood_send_morse,
 .vfo_op =  kenwood_vfo_op,
 .set_mem =  kenwood_set_mem,
 .get_mem =  kenwood_get_mem,
+.get_channel = ts570_get_channel,
+.set_channel = ts570_set_channel,
 .set_trn =  kenwood_set_trn,
 .get_trn =  kenwood_get_trn,
 .set_powerstat =  kenwood_set_powerstat,
@@ -589,7 +973,3 @@ const struct rig_caps ts570d_caps = {
 };
 
 
-/*
- * Function definitions below
- */
-
