@@ -2,7 +2,7 @@
  *  Hamlib Interface - toolbox
  *  Copyright (c) 2000-2005 by Stephane Fillod
  *
- *	$Id: misc.c,v 1.38 2005/04/09 09:49:12 fillods Exp $
+ *	$Id: misc.c,v 1.41 2006/11/02 15:19:58 aa6e Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -20,6 +20,16 @@
  *
  */
 
+/**
+ * \addtogroup rig_internal
+ * @{
+ */
+
+/**
+ * \file misc.c
+ * \brief Miscellaneous utility routines
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -38,8 +48,13 @@
 
 #include "misc.h"
 
-
-/*
+/**
+ * \brief Convert from binary to 4-bit BCD digits, little-endian
+ * \param bcd_data
+ * \param freq
+ * \param bcd_len
+ * \return bcd_data
+ *
  * Convert a long long (eg. frequency in Hz) to 4-bit BCD digits, 
  * packed two digits per octet, in little-endian order.
  * bcd_len is the number of BCD digits, usually 10 or 8 in 1-Hz units, 
@@ -71,7 +86,12 @@ HAMLIB_API to_bcd(unsigned char bcd_data[], unsigned long long freq, unsigned bc
 	return bcd_data;
 }
 
-/*
+/**
+ * \brief Convert BCD digits to a long long (eg. frequency in Hz)
+ * \param bcd_data
+ * \param bcd_len
+ * \return binary result (e.g. frequency)
+ *
  * Convert BCD digits to a long long (eg. frequency in Hz)
  * bcd_len is the number of BCD digits.
  *
@@ -95,7 +115,13 @@ unsigned long long HAMLIB_API from_bcd(const unsigned char bcd_data[], unsigned
 	return f;
 }
 
-/*
+/**
+ * \brief Convert binary to bcd, big-endian
+ * \param bcd_data
+ * \param freq
+ * \param bcd_len
+ * \return bcd_data
+ *
  * Same as to_bcd, but in Big Endian mode
  */
 unsigned char *
@@ -123,7 +149,12 @@ HAMLIB_API to_bcd_be(unsigned char bcd_data[], unsigned long long freq, unsigned
 	return bcd_data;
 }
 
-/*
+/**
+ * \brief Convert bcd to binary, big-endian
+ * \param bcd_data
+ * \param bcd_len
+ * \return binary value, e.g., frequency
+ *
  * Same as from_bcd, but in Big Endian mode
  */
 unsigned long long HAMLIB_API from_bcd_be(const unsigned char bcd_data[], unsigned bcd_len)
@@ -150,7 +181,11 @@ unsigned long long HAMLIB_API from_bcd_be(const unsigned char bcd_data[], unsign
 #define llabs(a) ((a)<0?-(a):(a))
 #endif
 
-/*
+/**
+ * \brief Pretty print a frequency
+ * \param str for result (may need up to 17 char)
+ * \param freq input in Hz
+ *
  * rig_freq_snprintf?
  * pretty print frequencies
  * str must be long enough. max can be as long as 17 chars
@@ -177,17 +212,11 @@ int HAMLIB_API sprintf_freq(char *str, freq_t freq)
 	return sprintf (str, "%g %s", f, hz);
 }
 
-const char * HAMLIB_API rig_strptrshift(rptr_shift_t shift)
-{
-	switch (shift) {
-	case RIG_RPT_SHIFT_MINUS: return "+";
-	case RIG_RPT_SHIFT_PLUS: return "-";
-
-	case RIG_RPT_SHIFT_NONE: return "None";
-	}
-	return NULL;
-}
-
+/**
+ * \brief Convert enum RIG_STATUS_... to printable string
+ * \param status RIG_STATUS_??
+ * \return string
+ */
 const char * HAMLIB_API rig_strstatus(enum rig_status_e status)
 {
 	switch (status) {
@@ -229,9 +258,16 @@ static struct {
 	{ RIG_MODE_ECSSLSB, "ECSSLSB" },
 	{ RIG_MODE_FAX, "FAX" },
 	{ RIG_MODE_NONE, "" },
+ 	{ RIG_MODE_SAM, "SAM" },
+ 	{ RIG_MODE_SAL, "SAL" },
+ 	{ RIG_MODE_SAH, "SAH" },
 };
 
-
+/**
+ * \brief Convert alpha string to enum RIG_MODE
+ * \param s input alpha string
+ * \return enum RIG_MODE_??
+ */
 rmode_t HAMLIB_API rig_parse_mode(const char *s)
 {
 	int i;
@@ -242,6 +278,11 @@ rmode_t HAMLIB_API rig_parse_mode(const char *s)
 	return RIG_MODE_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_MODE to alpha string
+ * \param mode RIG_MODE_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strrmode(rmode_t mode)
 {
 	int i;
@@ -256,7 +297,6 @@ const char * HAMLIB_API rig_strrmode(rmode_t mode)
 	return "";
 }
 
-
 static struct { 
 		vfo_t vfo;
 		const char *str;
@@ -272,6 +312,11 @@ static struct {
 		{ RIG_VFO_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to enum RIG_VFO_...
+ * \param s input alpha string
+ * \return RIG_VFO_...
+ */
 vfo_t HAMLIB_API rig_parse_vfo(const char *s)
 {
 	int i;
@@ -282,6 +327,11 @@ vfo_t HAMLIB_API rig_parse_vfo(const char *s)
 	return RIG_VFO_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_VFO_... to alpha string
+ * \param vfo RIG_VFO_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strvfo(vfo_t vfo)
 {
 	int i;
@@ -296,7 +346,6 @@ const char * HAMLIB_API rig_strvfo(vfo_t vfo)
 	return "";
 }
 
-
 static struct { 
 		setting_t func; 
 		const char *str;
@@ -314,7 +363,7 @@ static struct {
 	{ RIG_FUNC_AIP, "AIP" },
 	{ RIG_FUNC_MON, "MON" },
 	{ RIG_FUNC_MN, "MN" },
-	{ RIG_FUNC_RNF, "RNF" },
+	{ RIG_FUNC_RF, "RF" },
 	{ RIG_FUNC_ARO, "ARO" },
 	{ RIG_FUNC_LOCK, "LOCK" },
 	{ RIG_FUNC_MUTE, "MUTE" },
@@ -332,6 +381,11 @@ static struct {
 	{ RIG_FUNC_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to enum RIG_FUNC_...
+ * \param s input alpha string
+ * \return RIG_FUNC_...
+ */
 setting_t HAMLIB_API rig_parse_func(const char *s)
 {
 	int i;
@@ -343,6 +397,11 @@ setting_t HAMLIB_API rig_parse_func(const char *s)
 	return RIG_FUNC_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_FUNC_... to alpha string
+ * \param func RIG_FUNC_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strfunc(setting_t func)
 {
 	int i;
@@ -357,7 +416,6 @@ const char * HAMLIB_API rig_strfunc(setting_t func)
 	return "";
 }
 
-
 static struct { 
 		setting_t level;
 		const char *str;
@@ -393,6 +451,11 @@ static struct {
 	{ RIG_LEVEL_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to enum RIG_LEVEL_...
+ * \param s input alpha string
+ * \return RIG_LEVEL_...
+ */
 setting_t HAMLIB_API rig_parse_level(const char *s)
 {
 	int i;
@@ -404,6 +467,11 @@ setting_t HAMLIB_API rig_parse_level(const char *s)
 	return RIG_LEVEL_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_LEVEL_... to alpha string
+ * \param level RIG_LEVEL_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strlevel(setting_t level)
 {
 	int i;
@@ -418,7 +486,6 @@ const char * HAMLIB_API rig_strlevel(setting_t level)
 	return "";
 }
 
-
 static struct { 
 		setting_t parm;
 		const char *str;
@@ -432,6 +499,11 @@ static struct {
 	{ RIG_PARM_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to RIG_PARM_...
+ * \param s input alpha string
+ * \return RIG_PARM_...
+ */
 setting_t HAMLIB_API rig_parse_parm(const char *s)
 {
 	int i;
@@ -443,7 +515,11 @@ setting_t HAMLIB_API rig_parse_parm(const char *s)
 	return RIG_PARM_NONE;
 }
 
-
+/**
+ * \brief Convert enum RIG_PARM_... to alpha string
+ * \param parm RIG_PARM_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strparm(setting_t parm)
 {
 	int i;
@@ -458,7 +534,6 @@ const char * HAMLIB_API rig_strparm(setting_t parm)
 	return "";
 }
 
-
 static struct { 
 		vfo_op_t vfo_op;
 		const char *str;
@@ -479,6 +554,11 @@ static struct {
 	{ RIG_OP_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to enum RIG_OP_...
+ * \param s alpha string
+ * \return RIG_OP_...
+ */
 vfo_op_t HAMLIB_API rig_parse_vfo_op(const char *s)
 {
 	int i;
@@ -490,6 +570,11 @@ vfo_op_t HAMLIB_API rig_parse_vfo_op(const char *s)
 	return RIG_OP_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_OP_... to alpha string
+ * \param op RIG_OP_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strvfop(vfo_op_t op)
 {
 	int i;
@@ -504,7 +589,6 @@ const char * HAMLIB_API rig_strvfop(vfo_op_t op)
 	return "";
 }
 
-
 static struct { 
 		scan_t rscan;
 		const char *str;
@@ -520,6 +604,11 @@ static struct {
 	{ RIG_SCAN_NONE, "" },
 };
 
+/**
+ * \brief Convert alpha string to enum RIG_SCAN_...
+ * \param s alpha string
+ * \return RIG_SCAN_...
+ */
 scan_t HAMLIB_API rig_parse_scan(const char *s)
 {
 	int i;
@@ -529,10 +618,14 @@ scan_t HAMLIB_API rig_parse_scan(const char *s)
 			return scan_str[i].rscan;
 		}
 	}
-
 	return RIG_SCAN_NONE;
 }
 
+/**
+ * \brief Convert enum RIG_SCAN_... to alpha string
+ * \param rscan RIG_SCAN_...
+ * \return alpha string
+ */
 const char * HAMLIB_API rig_strscan(scan_t rscan)
 {
 	int i;
@@ -543,11 +636,29 @@ const char * HAMLIB_API rig_strscan(scan_t rscan)
 	for (i=0; scan_str[i].str != ""; i++)
 		if (rscan == scan_str[i].rscan)
 			return scan_str[i].str;
-
 	return "";
 }
 
+/**
+ * \brief convert enum RIG_RPT_SHIFT_... to printable character
+ * \param shift RIG_RPT_SHIFT_??
+ * \return alpha character
+ */
+const char * HAMLIB_API rig_strptrshift(rptr_shift_t shift)
+{
+	switch (shift) {
+	case RIG_RPT_SHIFT_MINUS: return "+";
+	case RIG_RPT_SHIFT_PLUS: return "-";
+	case RIG_RPT_SHIFT_NONE: return "None";
+	}
+	return NULL;
+}
 
+/**
+ * \brief Convert alpha char to enum RIG_RPT_SHIFT_...
+ * \param s alpha char
+ * \return RIG_RPT_SHIFT_...
+ */
 rptr_shift_t HAMLIB_API rig_parse_rptr_shift(const char *s)
 {
 	if (strcmp(s, "+") == 0)
@@ -558,3 +669,4 @@ rptr_shift_t HAMLIB_API rig_parse_rptr_shift(const char *s)
 		return RIG_RPT_SHIFT_NONE;
 }
 
+/** @} */
